@@ -254,7 +254,7 @@ umfit <- new("unmarkedFitPCO", fitType = "pcountOpen",
     call = match.call(), formula = formula, formlist = formlist, data = data,
     sitesRemoved=D$removed.sites, estimates = estimateList, AIC = fmAIC,
     opt = opt, negLogLike = fm$value, nllFun = nll, K = K, mixture = mixture,
-    dynamics = dynamics, immigration = immigration)
+    dynamics = dynamics, immigration = immigration, fix = fix)
 return(umfit)
 }
 
