@@ -2,7 +2,7 @@
 
 using namespace Rcpp ;
 
-SEXP nll_occu( SEXP yR, SEXP Xr, SEXP Vr, SEXP beta_psiR, SEXP beta_pR, SEXP ndR, SEXP knownOccR, SEXP navecR, SEXP X_offsetR, SEXP V_offsetR ) {
+SEXP nll_occu( SEXP yR, SEXP Xr, SEXP Vr, SEXP beta_psiR, SEXP beta_pR, SEXP ndR, SEXP knownOccR, SEXP navecR, SEXP X_offsetR, SEXP V_offsetR, SEXP link_psiR ) {
   arma::icolvec y = as<arma::icolvec>(yR);
   arma::mat X = as<arma::mat>(Xr);
   arma::mat V = as<arma::mat>(Vr);
@@ -13,12 +13,24 @@ SEXP nll_occu( SEXP yR, SEXP Xr, SEXP Vr, SEXP beta_psiR, SEXP beta_pR, SEXP ndR
   Rcpp::LogicalVector navec(navecR);
   arma::colvec X_offset = as<arma::colvec>(X_offsetR);
   arma::colvec V_offset = as<arma::colvec>(V_offsetR);
+  std::string link_psi = as<std::string>(link_psiR);
+  
   int R = X.n_rows;
   int J = y.n_elem / R;
-  arma::colvec logit_psi = X*beta_psi + X_offset;
+  
+  //Calculate psi
+  arma::colvec psi_lp = X*beta_psi + X_offset;
+  arma::colvec psi(R);
+  if(link_psi == "cloglog"){
+    psi = 1 - exp(-exp(psi_lp));
+  } else { 
+    psi = 1.0/(1.0+exp(-psi_lp));
+  }
+  
+  //Calculate p
   arma::colvec logit_p = V*beta_p + V_offset;
-  arma::colvec psi = 1.0/(1.0+exp(-logit_psi));
   arma::colvec p = 1.0/(1.0+exp(-logit_p));
+  
   double ll=0.0;
   int k=0; // counter
   for(int i=0; i<R; i++) {
